import { validateField, timeRemaining, formatSecondsNoHour } from './helper'
import { expect, describe, it } from '@jest/globals'

describe('Test validateField', () => {
  const profile = {
    firstName: 'Chicken',
    lastName: 'Saurus',
    email: 'chickensaurus@gmail.com',
    organization: 'PT Saurus Indonesia',
    phoneNumber: '+62838-091-738-20',
    pictureUrl:
      'https://www.freeiconspng.com/uploads/red-single-chicken-png-8.png',
    birthDate: '',
    gender: '',
    education: '',
    profession: '',
    industry: '',
    instagram: '',
    youtube: '',
    tiktok: '',
    phoneCode: ''
  }

  it('Return true if the field is not required', () => {
    expect(validateField('firstName', profile, true)).toBe(true)
  })

  it('Return true if the field is required and the value is exist', () => {
    expect(validateField('lastName', profile, false)).toBe(true)
  })

  it('Return false if the field is required but the value is missing', () => {
    expect(validateField('birthDate', profile, false)).toBe(false)
  })

  it('Return true for valid phone number', () => {
    expect(validateField('phoneNumber', profile, false)).toBe(true)
  })

  it('Return false for invalid phone number', () => {
    expect(
      validateField(
        'phoneNumber',
        { ...profile, phoneNumber: '+6211112222' },
        false
      )
    ).toBe(false)
  })

  it('Return true for valid phone code', () => {
    expect(
      validateField(
        'phoneCode',
        { ...profile, phoneCode: 'ID +62', phoneNumber: '838-091-738-20' },
        false
      )
    ).toBe(true)
  })

  it('Return false for invalid phone code', () => {
    expect(
      validateField(
        'phoneCode',
        { ...profile, phoneCode: 'ID +61', phoneNumber: '838-091-738-20' },
        false
      )
    ).toBe(false)
  })
})

describe('Test timeRemaining', () => {
  const strNow = new Date().toString()
  const strTime = (durationInMinutes: number): string => {
    const timeNow = new Date().getTime()
    const MS_PER_MINUTE = 60000
    return new Date(timeNow - durationInMinutes * MS_PER_MINUTE).toString()
  }

  it('Return 0 for empty timestamp', () => {
    expect(timeRemaining('')).toBe(0)
  })

  it('Return 0 for old time', () => {
    const oldTime = new Date('2000-01-01').toString()
    expect(timeRemaining(oldTime)).toBe(0)
  })

  it('Return 300 for current time', () => {
    expect(timeRemaining(strNow)).toBe(300)
  })

  it('Return 240 for 1 minute ago', () => {
    expect(timeRemaining(strTime(1))).toBe(240)
  })

  it('Return 120 for 3 minutes ago', () => {
    expect(timeRemaining(strTime(3))).toBe(120)
  })

  it('Return 0 for 5 minutes ago', () => {
    expect(timeRemaining(strTime(5))).toBe(0)
  })

  it('Return 0 for 6 minutes ago', () => {
    expect(timeRemaining(strTime(6))).toBe(0)
  })
})

describe('Test formatSecondsNoHour', () => {
  it('Return 00:00 for Infinity', () => {
    expect(formatSecondsNoHour(Infinity)).toBe('00:00')
  })

  it('Return 00:00 for 0 seconds', () => {
    expect(formatSecondsNoHour(0)).toBe('00:00')
  })

  it('Return 00:30 for 30 seconds', () => {
    expect(formatSecondsNoHour(30)).toBe('00:30')
  })

  it('Return 01:00 for 1 minute', () => {
    expect(formatSecondsNoHour(60)).toBe('01:00')
  })

  it('Return 01:30 for 90 seconds', () => {
    expect(formatSecondsNoHour(90)).toBe('01:30')
  })

  it('Return 03:00 for 3 minutes', () => {
    expect(formatSecondsNoHour(180)).toBe('03:00')
  })

  it('Return 05:00 for 5 minutes', () => {
    expect(formatSecondsNoHour(300)).toBe('05:00')
  })
})
