import * as React from 'react'
import * as SC from './styles'

import MediaQuery from 'react-responsive'
import i18n from './locale'
import EditedFieldInput from './component/EditedFieldInput'
import PhoneInput from './component/PhoneInput'
import ChangePassword from './component/ChangePassword'
import SocmedInputDefined from './component/SocmedInputDefined'

import instagramIcon from '../../assets/components/account/instagram.svg'
import youtubeIcon from '../../assets/components/account/youtube.svg'
import tiktokIcon from '../../assets/components/account/tiktok.svg'
import facebookIcon from '../../assets/components/account/facebook.svg'

import { FormEvent, useCallback, useEffect, useState } from 'react'
import { I18nextProvider, useTranslation } from 'react-i18next'
import { BiSolidEditAlt } from 'react-icons/bi'

import type {
  AccountProfileOptions,
  AccountProfileProps,
  Profile
} from './types'

const defaultOptions: AccountProfileOptions = {
  language: 'id',
  mode: 'light',
  fontLabel: 'Montserrat',
  buttonColor: 'var(--fair-green)',
  canEditPhoneNumber: true
}

export const Account: React.FC<AccountProfileProps> = ({
  data,
  options,
  onUpdate,
  onChangeNumber,
  onChangePassword
}) => {
  options = { ...defaultOptions, ...options }

  const {
    t,
    i18n: { changeLanguage }
  } = useTranslation()

  const { language } = options

  useEffect(() => {
    changeLanguage(language)
  }, [changeLanguage, language])

  const [profile, setProfile] = useState<Profile>({
    email: '',
    firstName: '',
    lastName: '',
    organization: '',
    phoneNumber: '',
    instagram: '',
    youtube: '',
    tiktok: ''
  })

  const [newProfile, setNewProfile] = useState<Profile>({
    ...profile
  })

  const [isEditing, setIsEditing] = useState(false)
  const [edited, setEdited] = useState(false)

  const initProfile = useCallback(() => {
    setProfile(data)
    setNewProfile({
      ...data
    })
  }, [data])

  useEffect(() => {
    initProfile()
  }, [initProfile])

  const handleEditClick = () => {
    setIsEditing(true)
  }

  const updateEdited = useCallback(() => {
    setEdited(JSON.stringify(newProfile) !== JSON.stringify(profile))
  }, [newProfile, profile])

  const handleProfileChange = useCallback(
    (event: FormEvent) => {
      const element = event.target as HTMLInputElement
      const key: keyof Profile = element.name as keyof Profile
      const temp = { ...newProfile }
      temp[key] = element.value
      setNewProfile(temp)
      updateEdited()
    },
    [newProfile, updateEdited]
  )

  const handleSave = () => {
    setIsEditing(false)
    onUpdate(newProfile)
  }

  const handleCancel = () => {
    initProfile()
    setEdited(false)
    setIsEditing(false)
  }

  return (
    <I18nextProvider i18n={i18n}>
      <SC.Main>
        <SC.Container>
          <SC.ProfileContainer>
            <SC.ProfileBox>
              <SC.FieldContainer>
                <SC.FieldLabel>{t('email')}</SC.FieldLabel>
                {isEditing ? (
                  <SC.FieldValue>
                    <SC.TextInput
                      name={'email'}
                      type='email'
                      value={newProfile.email}
                      disabled
                    />
                  </SC.FieldValue>
                ) : (
                  <SC.FieldValue>{newProfile.email}</SC.FieldValue>
                )}
              </SC.FieldContainer>
              <EditedFieldInput
                name={'firstName'}
                label={t('firstName')}
                type='text'
                value={newProfile.firstName}
                isEdit={isEditing}
                handleProfileChange={handleProfileChange}
              />
              <EditedFieldInput
                name={'lastName'}
                label={t('lastName')}
                type='text'
                value={newProfile.lastName}
                isEdit={isEditing}
                handleProfileChange={handleProfileChange}
              />
              <EditedFieldInput
                name={'organization'}
                label={t('organization')}
                type='text'
                value={newProfile.organization}
                isEdit={isEditing}
                handleProfileChange={handleProfileChange}
              />

              <PhoneInput
                inputText={{
                  id: 'phoneNumber',
                  label: t('phoneNumber')
                }}
                isEdit={isEditing}
                options={options}
                handleProfileChange={handleProfileChange}
                invalidFields={false}
                newProfile={newProfile}
                onChangeNumber={onChangeNumber}
              />

              <MediaQuery query='(min-width: 992px)'>
                <div style={{ padding: '16px 0px 16px 0px' }}>
                  <SC.Paragraph>
                    <SC.Text>{t('socialMedia')}</SC.Text>
                  </SC.Paragraph>
                </div>

                <EditedFieldInput
                  name={'instagram'}
                  label={t('instagram')}
                  type='text'
                  value={newProfile.instagram}
                  isSocmed={true}
                  socmedIcon={instagramIcon}
                  isEdit={isEditing}
                  handleProfileChange={handleProfileChange}
                />
                <EditedFieldInput
                  name={'youtube'}
                  label={t('youtube')}
                  type='text'
                  value={newProfile.youtube}
                  isSocmed={true}
                  socmedIcon={youtubeIcon}
                  isEdit={isEditing}
                  handleProfileChange={handleProfileChange}
                />
                <EditedFieldInput
                  name={'tiktok'}
                  label={t('tiktok')}
                  type='text'
                  value={newProfile.tiktok}
                  isSocmed={true}
                  socmedIcon={tiktokIcon}
                  isEdit={isEditing}
                  handleProfileChange={handleProfileChange}
                />

                <SocmedInputDefined
                  name={'facebook'}
                  label={t('facebook')}
                  type='text'
                  value={newProfile.facebook}
                  socmedIcon={facebookIcon}
                  isEdit={isEditing}
                  handleProfileChange={handleProfileChange}
                />

                {isEditing && (
                  <SC.ButtonContainer style={{ marginLeft: 'auto' }}>
                    <SC.CancelButton onClick={handleCancel}>
                      {t('cancel')}
                    </SC.CancelButton>
                    <SC.SaveButton
                      disabled={!edited}
                      style={{ marginLeft: '64px' }}
                      onClick={handleSave}
                    >
                      {t('save')}
                    </SC.SaveButton>
                  </SC.ButtonContainer>
                )}
              </MediaQuery>
            </SC.ProfileBox>
            {!isEditing && (
              <SC.ItemIcon>
                <BiSolidEditAlt onClick={handleEditClick} />
              </SC.ItemIcon>
            )}
          </SC.ProfileContainer>
          <SC.ChangePasswordContainer>
            <SC.ChangePasswordBox>
              <div style={{ padding: '0px 0px 18px 0px' }}>
                <SC.FieldLabel style={{ maxWidth: '100%' }}>
                  {t('changePassword')}
                </SC.FieldLabel>
              </div>

              <SC.FieldValue style={{ flexBasis: '100%', maxWidth: '100%' }}>
                <ChangePassword
                  options={options}
                  onChangePassword={onChangePassword}
                />
              </SC.FieldValue>
            </SC.ChangePasswordBox>
          </SC.ChangePasswordContainer>
          <MediaQuery query='(max-width: 991.98px)'>
            <SC.ProfileContainer>
              <SC.ProfileBox>
                <div style={{ padding: '0px 0px 16px 0px' }}>
                  <SC.Paragraph>
                    <SC.Text>{t('socialMedia')}</SC.Text>
                  </SC.Paragraph>
                </div>
                <EditedFieldInput
                  name={'instagram'}
                  label={t('instagram')}
                  type='text'
                  value={newProfile.instagram}
                  isSocmed={true}
                  socmedIcon={instagramIcon}
                  isEdit={isEditing}
                  handleProfileChange={handleProfileChange}
                />
                <EditedFieldInput
                  name={'youtube'}
                  label={t('youtube')}
                  type='text'
                  value={newProfile.youtube}
                  isSocmed={true}
                  socmedIcon={youtubeIcon}
                  isEdit={isEditing}
                  handleProfileChange={handleProfileChange}
                />
                <EditedFieldInput
                  name={'tiktok'}
                  label={t('tiktok')}
                  type='text'
                  value={newProfile.tiktok}
                  isSocmed={true}
                  socmedIcon={tiktokIcon}
                  isEdit={isEditing}
                  handleProfileChange={handleProfileChange}
                />

                <SocmedInputDefined
                  name={'facebook'}
                  label={t('facebook')}
                  type='text'
                  value={newProfile.facebook}
                  socmedIcon={facebookIcon}
                  isEdit={isEditing}
                  handleProfileChange={handleProfileChange}
                />
              </SC.ProfileBox>
            </SC.ProfileContainer>
            {isEditing && (
              <SC.ProfileContainer style={{ flexDirection: 'column' }}>
                <SC.FieldValue style={{ marginLeft: 'auto' }}>
                  <SC.ButtonContainer>
                    <SC.CancelButton onClick={handleCancel}>
                      {t('cancel')}
                    </SC.CancelButton>
                    <SC.SaveButton
                      disabled={!edited}
                      style={{ marginLeft: '64px' }}
                      onClick={handleSave}
                    >
                      {t('save')}
                    </SC.SaveButton>
                  </SC.ButtonContainer>
                </SC.FieldValue>
              </SC.ProfileContainer>
            )}
          </MediaQuery>
        </SC.Container>
      </SC.Main>
    </I18nextProvider>
  )
}
