import * as SC from '../styles'

import { useState, useEffect, useRef, FormEvent } from 'react'
import { useTranslation } from 'react-i18next'
import { useLocalStorage, useCountdown } from 'usehooks-ts'
import { timeRemaining, formatSecondsNoHour } from '../helper'

import type { AccountProfileOptions, Profile } from '../types'

interface Props {
  newProfile: Profile
  options: AccountProfileOptions
  onChangeNumber?: () => Promise<void>
}

const ChangeNumber: React.FC<Props> = ({
  newProfile,
  options: { fontLabel, buttonColor },
  onChangeNumber
}) => {
  const localStorageKey: string = `${newProfile.phoneNumber}::countdownChangePhoneNumber`

  const { t } = useTranslation()

  const [disableButton, setDisableButton] = useState(false)
  const [changeNumberRequested, setChangeNumberRequested] = useState(false)
  const [changeNumberSuccess, setChangeNumberSuccess] = useState(false)
  const [timeStamp, setTimeStamp] = useLocalStorage<string>(localStorageKey, '')
  const [countdownStart, setCountdownStart] = useState(timeRemaining(timeStamp))
  const [count, { startCountdown, resetCountdown }] = useCountdown({
    countStart: countdownStart,
    countStop: 0,
    intervalMs: 1000
  })

  const dialogRef = useRef(null)

  const handleShowModal = () => {
    if (dialogRef.current) {
      const element = dialogRef.current as HTMLDialogElement
      element.showModal()
    }
  }

  const handleCloseModal = () => {
    if (dialogRef.current) {
      const element = dialogRef.current as HTMLDialogElement
      element.close()
    }
    setDisableButton(false)
    setChangeNumberRequested(false)
  }

  const handleChangeNumber = async () => {
    setDisableButton(true)
    try {
      if (onChangeNumber) await onChangeNumber()
      setChangeNumberSuccess(true)
      resetCountdown()
      setTimeStamp(new Date().toString())
    } catch (error) {
      setChangeNumberSuccess(false)
    } finally {
      setDisableButton(false)
      setChangeNumberRequested(true)
    }
  }

  useEffect(() => {
    if (timeRemaining(timeStamp)) {
      startCountdown()
    }
    setCountdownStart(300)
  }, [timeStamp, startCountdown, setCountdownStart])

  return (
    <>
      {count ? (
        <SC.ChangeNumberContainer style={{ cursor: 'wait' }}>
          <SC.TextCountDown>{formatSecondsNoHour(count)}</SC.TextCountDown>
        </SC.ChangeNumberContainer>
      ) : (
        <SC.ChangeNumberContainer onClick={handleShowModal}>
          <SC.TextChangeNumber>{t('changeNumberText')}</SC.TextChangeNumber>
        </SC.ChangeNumberContainer>
      )}
      <SC.DialogConfirmation ref={dialogRef} onClick={handleCloseModal}>
        <SC.DialogContent
          onClick={(e: FormEvent) => {
            e.stopPropagation()
          }}
        >
          <SC.DialogTitle style={{ fontFamily: fontLabel }}>
            {t('changeNumberTitle')}
          </SC.DialogTitle>
          {changeNumberRequested ? (
            <SC.DialogResult>
              {changeNumberSuccess
                ? t('changeNumberSuccess')
                : t('changeNumberFailed')}
            </SC.DialogResult>
          ) : (
            <SC.DialogDescription>
              {t('changeNumberDescription')}
            </SC.DialogDescription>
          )}
          {changeNumberRequested ? (
            <SC.DialogActionSuccess>
              <SC.DialogActionButton
                style={{
                  width: '78px',
                  fontFamily: fontLabel,
                  color: 'white',
                  backgroundColor: buttonColor,
                  border: `1px solid ${buttonColor}`
                }}
                onClick={handleCloseModal}
              >
                Ok
              </SC.DialogActionButton>
            </SC.DialogActionSuccess>
          ) : (
            <SC.DialogAction
              style={{ width: '90%', margin: 'auto', marginTop: '40px' }}
            >
              <SC.DialogActionButton
                disabled={disableButton}
                style={{
                  width: '106px',
                  fontFamily: fontLabel,
                  color: buttonColor,
                  backgroundColor: 'var(--white)',
                  border: `1px solid ${buttonColor}`
                }}
                onClick={handleCloseModal}
              >
                {t('changeNumberCancel')}
              </SC.DialogActionButton>
              <SC.DialogActionButton
                disabled={disableButton}
                style={{
                  width: '106px',
                  fontFamily: fontLabel,
                  color: 'white',
                  backgroundColor: buttonColor,
                  border: `1px solid ${buttonColor}`
                }}
                onClick={handleChangeNumber}
              >
                {t('changeNumberConfirm')}
              </SC.DialogActionButton>
            </SC.DialogAction>
          )}
        </SC.DialogContent>
      </SC.DialogConfirmation>
    </>
  )
}

export default ChangeNumber
