import * as SC from './styles'
import i18next from '../../../lib/i18n'

import CardNumberIcon from '../../../assets/components/payment-profile/card-number.svg'
import IllustrationImage from '../../../assets/components/payment-profile/illustration-image.svg'
import IllustrationText from '../../../assets/components/payment-profile/illustration-text.svg'
import { BankLogoMap, BinMap } from '../bin/bin'

import { I18nextProvider } from 'react-i18next'
import { useTranslation } from 'react-i18next'
import { useState, useEffect } from 'react'

import type {
  PaymentProfiles,
  PaymentProfileListProps,
  PaymentProfileListOptions
} from './types'

const defaultOptions: PaymentProfileListOptions = {
  language: 'id',
  isAddedSetDefault: true,
  titleFont: 'Montserrat',
  contentFont: 'Poppins',
  actionColor: '#56ab2f',
  disableColor: '#d0ecb4',
  boxShadow: '0px 15px 30px -5px rgba(86, 171, 47, 0.15)',
  selectedCardBackground: '#EBFAD6',
  selectedCardBorder: '2px solid #56AB2F',
  selectable: false
}

export const PaymentProfileList: React.FC<PaymentProfileListProps> = ({
  paymentProfiles,
  options,
  onSelectProfile,
  onSetDefault
}) => {
  options = { ...defaultOptions, ...options }

  const {
    t,
    i18n: { changeLanguage }
  } = useTranslation('paymentProfileList')

  const {
    language,
    isAddedSetDefault,
    titleFont,
    contentFont,
    actionColor,
    disableColor,
    boxShadow,
    selectedCardBackground,
    selectedCardBorder,
    selectable
  } = options

  useEffect(() => {
    changeLanguage(language)
  }, [changeLanguage, language])

  if (typeof paymentProfiles === 'string') {
    paymentProfiles = JSON.parse(paymentProfiles)
  }

  const defaultCardBackground = '#FFFFFF'
  const defaultCardBorder = '2px solid transparent'

  const [selected, setSelected] = useState(-1)

  const getBankLogo = (masked_card: string) => {
    if (masked_card.length >= 6) {
      const binDigits = masked_card.slice(0, 6)
      const bank = BinMap[Number(binDigits)]
      if (bank) {
        return BankLogoMap[bank]
      }
    }
    return CardNumberIcon
  }

  const _dispatchSelectProfile = (item: PaymentProfiles, index: number) => {
    if (selectable) {
      setSelected(index)
      onSelectProfile(item)
    }
  }

  const _dispatchSetDefault = (item: PaymentProfiles) => {
    onSetDefault(item)
  }

  return (
    <I18nextProvider i18n={i18next}>
      {paymentProfiles && paymentProfiles.length > 0 ? (
        <>
          {(paymentProfiles as PaymentProfiles[]).map(
            (item: PaymentProfiles, index: number) => (
              <SC.PaymentCard
                style={{
                  boxShadow: boxShadow,
                  cursor: selectable ? 'pointer' : '',
                  background:
                    selected === index
                      ? selectedCardBackground
                      : defaultCardBackground,
                  border:
                    selected === index ? selectedCardBorder : defaultCardBorder
                }}
                onClick={() => _dispatchSelectProfile(item, index)}
              >
                <SC.PaymentInfoContainer>
                  {selectable && (
                    <SC.SelectProfileButton
                      type='radio'
                      checked={selected === index}
                    />
                  )}
                  {item.image.length > 0 ||
                  getBankLogo(item.masked_card) === CardNumberIcon ? (
                    <SC.PaymentLogoContainer>
                      {getBankLogo(item.masked_card) === CardNumberIcon ? (
                        <SC.PaymentDefault
                          src={
                            item.image.length
                              ? item.image
                              : getBankLogo(item.masked_card)
                          }
                          alt={`card-${index}`}
                        />
                      ) : (
                        <SC.PaymentLogo
                          src={
                            item.image.length
                              ? item.image
                              : getBankLogo(item.masked_card)
                          }
                          alt={`card-${index}`}
                        />
                      )}
                    </SC.PaymentLogoContainer>
                  ) : (
                    <>
                      {getBankLogo(item.masked_card) === CardNumberIcon ? (
                        <SC.PaymentDefault
                          src={
                            item.image.length
                              ? item.image
                              : getBankLogo(item.masked_card)
                          }
                          alt={`card-${index}`}
                        />
                      ) : (
                        <SC.PaymentLogo
                          src={
                            item.image.length
                              ? item.image
                              : getBankLogo(item.masked_card)
                          }
                          alt={`card-${index}`}
                        />
                      )}
                    </>
                  )}
                  <SC.PaymentInfo>
                    <SC.InfoCard style={{ fontFamily: titleFont }}>
                      <SC.CardTitle>{item.name}</SC.CardTitle>
                      {isAddedSetDefault && (
                        <>
                          {item.is_default && (
                            <SC.TextDefault>(Default)</SC.TextDefault>
                          )}
                        </>
                      )}
                    </SC.InfoCard>
                    <SC.InfoUser style={{ fontFamily: contentFont }}>
                      {item.masked_card}
                    </SC.InfoUser>
                    <SC.PmPt16 style={{ fontFamily: titleFont }}>
                      {t('cardName')}
                    </SC.PmPt16>
                    <SC.InfoUser style={{ fontFamily: contentFont }}>
                      {item.card_holder}
                    </SC.InfoUser>
                  </SC.PaymentInfo>
                </SC.PaymentInfoContainer>
                {isAddedSetDefault && (
                  <SC.PaymentBtnDefault>
                    {item.is_default ? (
                      <SC.BtnInactive
                        style={{
                          fontFamily: titleFont,
                          border: `1px solid ${disableColor}`,
                          color: `${disableColor}`
                        }}
                      >
                        {t('setDefaultText')}
                      </SC.BtnInactive>
                    ) : (
                      <SC.BtnDefault
                        style={{
                          fontFamily: titleFont,
                          border: `1px solid ${actionColor}`,
                          color: `${actionColor}`
                        }}
                        onClick={() => _dispatchSetDefault(item)}
                      >
                        {t('setDefaultText')}
                      </SC.BtnDefault>
                    )}
                  </SC.PaymentBtnDefault>
                )}
              </SC.PaymentCard>
            )
          )}
        </>
      ) : (
        <>
          <SC.PaymentIllustrationBox>
            <div>
              <img src={IllustrationImage} />
            </div>
            <div>
              <img src={IllustrationText} />
            </div>
          </SC.PaymentIllustrationBox>
          <SC.PaymentIllustrationText>
            <SC.PaymentIllustrationTitle style={{ fontFamily: titleFont }}>
              {t('titleEmptyPayment')}
            </SC.PaymentIllustrationTitle>
            <SC.PaymentIllustrationDescription
              style={{ fontFamily: contentFont }}
            >
              {t('descriptionAddPayment')}
            </SC.PaymentIllustrationDescription>
          </SC.PaymentIllustrationText>
        </>
      )}
    </I18nextProvider>
  )
}

export default PaymentProfileList
